package example;

/**
 * This class processes a text file that specifies a set of properties, one per line. Each property has the syntax
 * 'key'='value'. The input file is processed, ordered by key and then the ordered result is stored in another
 * text file.
 */

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class OrderPropertyFile {

    private String inputFileName, outputFileName;

    /**
     * Creates a OrderPropertyFile object, specifying the file to process and the file to store the result.
     * 
     * @param inputFileName the name of the file containing the properties to order
     * @param outputFileName the name of the file where the result of ordering the properties is stored.
     */
    public OrderPropertyFile(String inputFileName, String outputFileName) {
        this.inputFileName = inputFileName;
        this.outputFileName = outputFileName;
    }

    /**
     * Process input file and returns a list of Pair objects representing the content of the input file.
     * 
     * @return a list of Pair objects.
     */
    public List<Pair> readPairs() {

        List<Pair> data = new ArrayList<Pair>();

        System.err.println(">>> Opening file for processing: " + inputFileName);

        try{
            // FileReader is a character oriented input stream (reader) defined in java.io
            // LineNumberReader is another reader, that allows line to line processing
            LineNumberReader lnrEntrada = new LineNumberReader( new FileReader(inputFileName));
            String[] pair;
            String line;

            while( (line = lnrEntrada.readLine()) != null) {
                pair = line.split("=");
                if( pair.length != 2)
                    throw new SyntaxException(line);

                data.add( new Pair( pair[0], pair[1]));
            }

        // if an exception is checked (extends java.lang.Exception), then the compiler enforces
        // the corresponding catch clause
        // if an exception is unchecked (extends java.lang.RuntimeException, then the compiler
        // does not enforce a corresponding catch clause. However, it can be done.
        } catch(SyntaxException se){
            System.err.println("Error processing file. Invalid line: '" + se.getErrorLine() + "'");
            System.exit(1);
        } catch(IOException io){
            System.err.println("Error opening file: " +  inputFileName);
            System.exit(1);
        }

        return data;
    }

    /**
     * @param l the list of pairs to write
     * 
     * Write the list of Pair objects into the output file of this object.
     */
    public void writePairs(List<Pair> l) {
        try{

            // FileWriter is a character oriented output stream (writer)
            FileWriter out = new FileWriter( outputFileName);

            // Iterator is an interface that allows easy iteration over a collection of objects
            Iterator<Pair> it = l.iterator();
            while( it.hasNext() ) {
                out.write(it.next()+"\n");
            }

            System.err.println(">>> Closing ordered file " + outputFileName);
            out.close();

        } catch(IOException io){
            System.err.println("Error while writing file: " + io.getMessage());
            System.exit(1);
        }
    }
    
    /**
     * @param args a vector with the names of the input and output files.
     * 
     * Reads the input file and writes it sorted in the output file.
     */
    public static void main(String[] args) {

        // validate input arguments
        if (args.length < 2) {
            System.out.println("Insufficient arguments. <java> Program input_file output_file");
            return;
        }
        List<Pair> listOfPairs = null;

        // read, sort and write file
        OrderPropertyFile order = new OrderPropertyFile(args[0], args[1]);

        listOfPairs = order.readPairs();
        Collections.sort(listOfPairs);
        order.writePairs(listOfPairs);
    }
}
